function [xc, his, xrec] = levmar(xc,fun,tol,maxit)
% Levenberg-Marquardt, trust region control of LM parameter
%
% Input: xc = initial iterate.  n x 1
%        fun= objective function handle,  [f, g, H] = fun(x)
%             f: 1 x 1;  g: n x 1;  H: n x n
%        tol= termination criterion norm(grad) < tol
%        maxit= maximum iterations (optional) default = 100
%
% Output: xc  = solution
%         his = iteration history, each column is
%       [norm(grad), fun, number of stepsize cuts]'
%         xrec  n x Niteration, x's trace
%---------------------------
% C. T. Kelley, Dec 14, 1997
% Y. Huang,     Oct 06, 2011

if nargin < 4,  maxit = 100;   end
nu0 = 1e-3;
k   = 1;
[fc,gc,H] = fun(xc);    norm_gc = norm(gc);
nu  = 0.5;  % nu is unitless in my code
n   = numel(xc);
if nargout > 1,  his = zeros(3, maxit);   end
if nargout > 2,  xrec= zeros(n, maxit);   end

while(norm_gc > tol && k <= maxit)
  if nargout > 2,  xrec(:,k) = xc;   end
  hc= H + nu*diag(diag(H));   % (my modification)
  dc= hc\ gc;
  xt= xc- dc;
  [xc, nu, idid] = trtestlm(fun,xc,xt, fc,gc,H, nu,nu0);
  if nargout > 1,  his(:,k)= [norm_gc;  fc;  idid];   end
  [fc,gc,H] = fun(xc);    norm_gc = norm(gc);
  k = k + 1;
end
if nargout > 1, his(:,k:end)  = [];  end
if nargout > 2, xrec(:, k:end)= [];  end
return

%--------------------------------
function [z, nu, k] = trtestlm(fun,xc,xt, fc,gc,H, nu,nu0)
%   Input:
% xc: current point;  xt: trial next point
% fc,gc,H: belong to xc
% nu: current nu
%   Output:
% z:  next point;  nu: adapted nu;  k: # trust region tests

mu0 = .1;  mulow = .25;  muhigh = .75;  mup = 2; mdown = .5;
z   = xc;
for k = 1: 20
  ft  = fun(xt);
  s   = xt-xc;  ared = fc-ft;  pred = -0.5*gc'*s;  % a bug was in Kelley's code
  rat = ared/pred;             % quantifies how well local landspace is modeled by a quadratic
  if (rat < mu0)               % poor quadratic modeling 
    nu= max(nu*mup, nu0);      % shrink trust region
    hc= H + nu*diag(diag(H));  % (my modification)
    dc= hc\ gc;
    xt= xc- dc;                % make new xt
  elseif (rat < mulow)         % quadratic modeling not good
    nu= max(nu*mup, nu0);      % shrink trust region
    z = xt;                    % xt acceptable
    break
  else                         % quadratic modeling ok
    z = xt;                    % keep xt and trust region
    if (rat > muhigh)          % quadratic modeling terrific
      nu = mdown * nu;         % expand trust region
      if nu < nu0,  nu=0;  end % allow Newton fast convergence
    end
    break
  end
end
return
%==============================
% driver script follows:

clear all
x0  = [-1; 2.9];    % inital point
[xc, his, xrec] = levmar(x0, @Rosenbrock_optim, 1e-4, 50);
Nit = size(his, 2); % # of iterations

figure
subplot(2,2,1)
plot_2DstateTrace([xrec, xc], @Rosenbrock, [-2 2 -1 3])
title('Levenberg-Marquardt code, adaptive trust region')

subplot(2,2,2)
plot((1:Nit), his(3,:), 'b-', 'linewidth',2), hold on
plot((1:Nit), log10(his(2,:)./his(2,1)), 'r-', 'linewidth',2)
xlabel('iteration step')
legend('# trust region tests', 'log(misfit)', 'Location', 'East')
% print -djpeg90 LevenMarqu.jpg

