function [lhat, trunc] = my_polymod(q0, qp0, lamc, qc, blow, bhigh, lamm, qm)
% Cubic/quadratic linesearch
%
% Finds minimizer lambda 'lhat' of the cubic polynomial q on the interval
% [blow * lamc, bhigh * lamc], such that
% q(0) = q0,  q'(0) = qp0,  q(lamc) = qc,  q(lamm) = qm
%
% If data for a cubic is not available (first stepsize reduction) then
% q is the quadratic such that
% q(0) = q0,  q'(0) = qp0,  q(lamc) = qc
% 
% trunc  0/1  1 if lhat value is truncated by the interval boundary.
%
% Ref:
% [1] C.T. Kelley, "Iterative Methods for Optimization", Section 3.2.1
% [2] Nocedal & Wright, "Numerical Optimization", 2nd Ed., Eq.3.58

if (nargin < 7)  % quadratic model
  lhat= -qp0 *lamc*lamc /(2*(qc - q0 - qp0*lamc));
else             % cubic model
  a= [lamc^2, lamc^3; lamm^2, lamm^3];
  b= [qc; qm] - q0 - qp0 * [lamc; lamm];
  c= a\b;
  lhat= (-c(1) + sqrt(c(1)*c(1) - 3* c(2) *qp0)) / (3*c(2));
end
lleft = lamc * blow;
lright= lamc * bhigh;
trunc = 0;
if (lhat < lleft),  lhat = lleft;  trunc = 1;  end
if (lhat > lright), lhat = lright; trunc = 1;  end
return
%===================
% test:
% y = a x^2 + bx
a = 0.5;   b = -2.36;
x = [0, 3];
y = a*x.^2 + b*x;
xhat = x(1) + my_polymod(y(1), 2*a*x(1)+b, x(2)-x(1), y(2), -inf, inf);
figure
t = linspace(-1, 4, 50);
plot(t, a*t.^2 + b*t), hold on
plot([xhat, xhat], [-1,4], 'r')

% y = a x^3 + b x^2 + c x
a = 1/3;   b = -3;  c = 1;
x = [0, 3, 4];
y = a*x.^3 + b*x.^2 + c*x;
xhat = x(1) + my_polymod(y(1), 3*a*x(1)^2+2*b*x(1)+c, x(2)-x(1), y(2), ...
            -inf, inf, x(3)-x(1), y(3));
figure
t = linspace(-2, 10, 50);
plot(t, a*t.^3 + b*t.^2 + c*t), hold on
plot([xhat, xhat], [-5,16], 'r')
