function [xc, his, xrec] = steep_Armijo(x0, fun, tol, maxit, MOD)
% Steepest descent w/ Armijo rule & polynomial linesearch
% x0    n x 1   initial
% fun   objective function handle
%       [f,g] = fun(x), where f is scalar, g = Grad fun(x) is n x 1
% tol   termination criterion norm(grad) < tol; default = 1e-5
% maxit maximum # iterations, default = 500
% MOD   [0]  Kelly's method of choosing initial lambda
%        1   analytical step length for choosing initial lambda
%        2   using only diag(Hessian) for 'analytical' step length as in 1
%--------------
% xc   n x 1    solution
% xrec n x Nit  recorded trace (optional), not including the output xc 
% his  3 x Nit  history, each column: [f;  nvAC;  ntru], where
%         f     function value,  not including the one @ output xc
%         nvAC  # violated Armijo Conditions per iteration
%         ntru  # polynomial predicted step length is truncated by preset interval
% Requires: polymod.m
% Ref:
% [1] C.T. Kelley, "Iterative Methods for Optimization"

c1   = 1e-4;  % Armijo coefficient
blow = .1;  bhigh = .8; 
if (nargin < 3 || isempty(tol)),    tol  = 1e-5;  end
if (nargin < 4 || isempty(maxit)),  maxit= 500;   end
if (nargin < 5 || isempty(MOD)),    MOD  = 0;     end

n = numel(x0);
if nargout > 1
  his = zeros(3, maxit);
  if nargout > 2, xrec = zeros(n, maxit);  end
end

k  = 1;  
xc = x0;   [fc,gc, H] = fun(xc);   norm_gc = norm(gc);
while(norm_gc > tol &&  k <= maxit)
  if (nargout > 2), xrec(:,k) = xc;  end
  narm   = 0;   %  # violate Armijo
  ntrunc = 0;   %  # polynom. prediction out of bound
  if (MOD==0)   %  fixup for very long steps:  ([1] Section 3.4)
    lambda = min(1,25/(1+norm_gc));
  elseif (MOD==1)
    lambda = (gc'*gc) / (gc'*H*gc); % analytic step length as initial step
  else
    lambda = (gc'*gc) / sum(diag(H).*gc.*gc);  % use diag(H) only
  end
  xt = xc - lambda*gc;  ft = fun(xt);
  fgoal  = fc - c1 * lambda*(gc'*gc);
  %       for polynomial line search:
  q0 = fc;  qp0 = -gc'*gc;  lamc = lambda;  qc = ft;
  while(ft > fgoal)   % Armijo violated
    narm = narm + 1;
    if(narm > 10), disp(' Armijo error in steepest descent ')
      if nargout > 1
        his(:, k) = [fc; narm; ntrunc]; 
        his(:, k+1:end)=[];
        if nargout > 2, xrec(:, k+1:end)=[];  end
      end
      return
    end
    if (narm == 1)
      [lambda, itru] = my_polymod(q0, qp0, lamc, qc, blow, bhigh);
    else
      [lambda, itru] = my_polymod(q0, qp0, lamc, qc, blow, bhigh, lamm, qm);
    end
    ntrunc = ntrunc + itru;
    qm = qc;  lamm = lamc;  lamc = lambda;
    xt = xc - lambda*gc;
    ft = fun(xt);  qc = ft;
    fgoal = fc - c1 * lambda*(gc'*gc);
  end
  if (nargout > 1),  his(:,k) = [fc; narm; ntrunc];  end
  k  = k + 1;
  xc = xt;   [fc,gc, H] = fun(xc);   norm_gc = norm(gc);
end
return
%===================
